@@ -35,19 +35,19 @@
 
 ;;; These variables are here because many things depend on them
 (defvar-local idris-buffer-dirty-p t
-  "An Idris buffer is dirty if there have been modifications since it was last loaded")
+  "An Idris buffer is dirty if it have been modified since it was last loaded.")
 
 (defvar idris-currently-loaded-buffer nil
-  "The buffer currently loaded by the running Idris")
+  "The buffer currently loaded by the running Idris.")
 
 (defvar idris-loaded-region-overlay nil
-  "The region loaded by Idris, should such a thing exist")
+  "The region loaded by Idris, should such a thing exist.")
 
 (defvar idris-process-current-working-directory ""
-  "Working directory of Idris process")
+  "Working directory of Idris process.")
 
 (defvar idris-command-line-option-functions nil
-  "A list of functions to call to compute the command-line arguments to Idris.
+  "A list of functions to call to compute the `command-line' arguments to Idris.
 Each function should take no arguments and return a list of
 strings that are suitable arguments to `start-process'.")
 
@@ -71,8 +71,15 @@ Lisp package.")
                (get-buffer buffer))
               ((bufferp buffer)
                buffer)
-              (t (message "don't know how to kill buffer")))))
-    (when (and buf (buffer-live-p buf)) (kill-buffer buf))))
+              (t (message "don't know how to kill buffer"))))
+        (return-buffer (seq-find (lambda (b) (eq 'idris-mode (buffer-local-value 'major-mode b)))
+                                 (buffer-list))))
+    (when (and buf (buffer-live-p buf))
+      (let ((win (get-buffer-window buf)))
+        (kill-buffer buf)
+        (when (and (null (window-prev-buffers win)) (< 1 (length (window-list))))
+          (delete-window win))))
+    (when return-buffer (pop-to-buffer return-buffer `(display-buffer-reuse-window)))))
 
 (defun idris-minibuffer-respecting-message (text &rest args)
   "Display TEXT as a message, without hiding any minibuffer contents."
@@ -89,7 +96,7 @@ Lisp package.")
 (defmacro idris-save-marker (marker &rest body)
   "Save the contents of the marker MARKER while executing BODY."
   (declare (indent 1))
-  (let ((pos (cl-gensym "pos")))
+  (let ((pos (gensym "pos")))
   `(let ((,pos (marker-position ,marker)))
      (prog1 (progn . ,body)
        (set-marker ,marker ,pos)))))
@@ -99,16 +106,16 @@ Lisp package.")
 More precisely, PROPS are added to the region between the point's
 positions before and after executing BODY."
   (declare (indent 1))
-  (let ((start (cl-gensym)))
+  (let ((start (gensym "foo")))
     `(let ((,start (point)))
        (prog1 (progn ,@body)
-	 (add-text-properties ,start (point) ,props)))))
+         (add-text-properties ,start (point) ,props)))))
 
 (defmacro idris-propertize-spans (spans &rest body)
   "Execute BODY and add the properties indicated by SPANS to the
 inserted text (that is, relative to point prior to insertion)."
   (declare (indent 1))
-  (let ((start (cl-gensym)))
+  (let ((start (gensym)))
     `(let ((,start (point)))
        (prog1 (progn ,@body)
          (cl-loop for (begin length props) in ,spans
@@ -133,7 +140,7 @@ inserted text (that is, relative to point prior to insertion)."
   "The decors that should light up as responsive to mouse clicks.")
 
 (defun idris-semantic-properties-face (props)
-  "Compute the text property `face' from the Idris properties for a region."
+  "Compute the text property `face' from the Idris properties (PROPS) for a region."
   (let* ((decor (assoc :decor props))
          (implicit (assoc :implicit props))
          (text-format (assoc :text-formatting props))
@@ -160,16 +167,13 @@ inserted text (that is, relative to point prior to insertion)."
          (err-face (if idris-err
                        '(idris-warning-face)
                      ()))
-         (text-face (pcase text-format
-                      (`(:text-formatting :bold)
-                       '(bold))
-                      (`(:text-formatting :italic)
-                       '(italic))
-                      (`(:text-formatting :underline)
-                       '(underline))
+         (text-face (pcase (cadr text-format)
+                      (:bold      '(bold))
+                      (:italic    '(italic))
+                      (:underline '(underline))
                       (_ nil)))
          (link-face (if link-href '(idris-link-face) ()))
-         (unique-val (cl-gensym)) ; HACK to stop consecutive mouse-faces from interfering
+         (unique-val (gensym)) ; HACK to stop consecutive mouse-faces from interfering
          (mousable-face
           (cond ((member (cadr decor) idris-semantic-properties-clickable-decors)
                  `((:inherit (,decor-face highlight) :hack ,unique-val)))
@@ -192,7 +196,7 @@ inserted text (that is, relative to point prior to insertion)."
 
 
 (defun idris-semantic-properties-eldoc (props)
-  "Compute an Eldoc string from Idris semantic properties PROPS"
+  "Compute an Eldoc string from Idris semantic properties (PROPS)."
   (let* ((name (assoc :name props))
          (namespace (assoc :namespace props))
          (source-file (assoc :source-file props))
@@ -250,7 +254,7 @@ inserted text (that is, relative to point prior to insertion)."
           (t nil))))
 
 (defun idris-semantic-properties (props)
-  "Compute how to highlight with Idris compiler properties PROPS."
+  "Compute how to highlight with Idris compiler properties (PROPS)."
   (let* ((name (assoc :name props))
          (decor (assoc :decor props))
          (term (assoc :tt-term props))
@@ -289,7 +293,7 @@ inserted text (that is, relative to point prior to insertion)."
             (if term
                 (list 'idris-tt-term (cadr term))
               ())
-            (if key
+            (if (and key (not (string-empty-p (cadr key))))
                 (list 'idris-name-key (concat "{{{{{" (cadr key) "}}}}}"))
               ())
             (if idris-err
@@ -343,27 +347,27 @@ The list of patterns is searched for a HEAD `eq' to the car of
 VALUE. If one is found, the BODY is executed with ARGS bound to the
 corresponding values in the CDR of VALUE."
   (declare (indent 1))
-  (let ((operator (cl-gensym "op-"))
-	(operands (cl-gensym "rand-"))
-	(tmp (cl-gensym "tmp-")))
+  (let ((operator (gensym "op-"))
+        (operands (gensym "rand-"))
+        (tmp (gensym "tmp-")))
     `(let* ((,tmp ,value)
-	    (,operator (car ,tmp))
-	    (,operands (cdr ,tmp)))
+            (,operator (car ,tmp))
+            (,operands (cdr ,tmp)))
        (cl-case ,operator
-	 ,@(mapcar (lambda (clause)
+         ,@(mapcar (lambda (clause)
                      (if (eq (car clause) t)
                          `(t ,@(cdr clause))
                        (cl-destructuring-bind ((op &rest rands) &rest body) clause
                          `(,op (cl-destructuring-bind ,rands ,operands
                                  . ,body)))))
-		   patterns)
-	 ,@(if (eq (caar (last patterns)) t)
-	       '()
-	     `((t (error "ELISP destructure-case failed: %S" ,tmp))))))))
+                   patterns)
+         ,@(if (eq (caar (last patterns)) t)
+               '()
+             `((t (error "ELISP destructure-case failed: %S" ,tmp))))))))
 
 (defun idris-lidr-p (&optional buffer)
-  "Return t if BUFFER is a literate Idris file, or nil otherwise. Use the current buffer if
-BUFFER is not supplied or is nil."
+  "Return t if BUFFER is a literate Idris file, or nil otherwise.
+Use the current buffer if BUFFER is not supplied or is nil."
   (let ((file-name (buffer-file-name buffer)))
     ;; We check for nil here because idris-lidr-p might be called on
     ;; buffers that don't have associated files, such as the REPL
@@ -379,7 +383,7 @@ BUFFER is not supplied or is nil."
     (overlay-put overlay 'help-echo help-echo)))
 
 (defun idris-clear-file-link-overlays (&optional mode)
-  "Remove all file link overlays from the current buffer"
+  "Remove all file link overlays from the current buffer."
   (when (or (not mode) (eq major-mode mode))
     (remove-overlays (point-min) (point-max) 'idris-file-link t)))
 
@@ -404,4 +408,53 @@ relative to SRC-DIR"
         (when (file-exists-p lidr)
           (make-link lidr))))))
 
+(defvar idris-protocol-version 0 "The protocol version.")
+(defvar idris-protocol-version-minor 0 "The protocol minor version.")
+
+(defun >=-protocol-version (major minor)
+  (or  (> idris-protocol-version major)
+       (and (>= idris-protocol-version       major)
+            (>= idris-protocol-version-minor minor))))
+
+(defun idris-get-line-num (position)
+  "Get the absolute line number at POSITION."
+  ;; In Emacs 26.1 > line-number-at-pos accepts
+  ;; additional optional argument ABSOLUTE which
+  ;; removes need for `save-restriction' and `widen'
+  (save-restriction
+    (widen)
+    (line-number-at-pos position)))
+
+(defun idris-operator-at-position-p (pos)
+  "Return t if syntax lookup is `.' or char after POS is `-'."
+  (or (equal (syntax-after pos) (string-to-syntax "."))
+      (eq (char-after pos) ?-)))
+
+(defun idris-thing-at-point ()
+  "Return the line number and name at point as a cons.
+Use this in Idris source buffers."
+  (let ((line (idris-get-line-num (point))))
+    (cons
+     (if (idris-operator-at-position-p (point))
+         (save-excursion
+           (skip-syntax-backward ".")
+           (let ((beg (point)))
+             (skip-syntax-forward ".")
+             (buffer-substring-no-properties beg (point))))
+       ;; Try if we're on a symbol or fail otherwise.
+       (or (current-word t)
+           (user-error "Nothing identifiable under point")))
+     line)))
+
+(defun idris-name-at-point ()
+  "Return the name at point, taking into account semantic annotations.
+Use this in Idris source buffers or in compiler-annotated output.
+Does not return a line number."
+  (let ((ref (cl-remove-if
+              #'null
+              (cons (get-text-property (point) 'idris-ref)
+                    (cl-loop for overlay in (overlays-at (point))
+                             collecting (overlay-get overlay 'idris-ref))))))
+    (car (or ref (idris-thing-at-point)))))
+
 (provide 'idris-common-utils)
